@@ -1,6 +1,6 @@
 #!/usr/bin/env python3 -i
 #
-# Copyright 2013-2021 The Khronos Group Inc.
+# Copyright 2013-2022 The Khronos Group Inc.
 #
 # SPDX-License-Identifier: Apache-2.0
 """Base class for source/header/doc generators, as well as some utility functions."""
@@ -43,7 +43,10 @@ def enquote(s):
     """Return string argument with surrounding quotes,
       for serialization into Python code."""
     if s:
-        return "'{}'".format(s)
+        if isinstance(s, str):
+            return "'{}'".format(s)
+        else:
+            return s
     return None
 
 
@@ -52,10 +55,13 @@ def regSortCategoryKey(feature):
     Sorts by category of the feature name string:
 
     - Core API features (those defined with a `<feature>` tag)
+        - (sort VKSC after VK)
     - ARB/KHR/OES (Khronos extensions)
     - other       (EXT/vendor extensions)"""
 
     if feature.elem.tag == 'feature':
+        if feature.name.startswith('VKSC'):
+            return 0.5
         return 0
     if (feature.category == 'ARB'
         or feature.category == 'KHR'
@@ -119,8 +125,11 @@ class GeneratorOptions:
                  removeExtensions=None,
                  emitExtensions=None,
                  emitSpirv=None,
+                 emitFormats=None,
                  reparentEnums=True,
-                 sortProcedure=regSortFeatures):
+                 sortProcedure=regSortFeatures,
+                 requireCommandAliases=False,
+                ):
         """Constructor.
 
         Arguments:
@@ -152,6 +161,8 @@ class GeneratorOptions:
         to None.
         - emitSpirv - regex matching names of extensions and capabilities
         to actually emit interfaces for.
+        - emitFormats - regex matching names of formats to actually emit
+        interfaces for.
         - reparentEnums - move <enum> elements which extend an enumerated
         type from <feature> or <extension> elements to the target <enums>
         element. This is required for almost all purposes, but the
@@ -217,6 +228,10 @@ class GeneratorOptions:
         """regex matching names of extensions and capabilities
         to actually emit interfaces for."""
 
+        self.emitFormats = self.emptyRegex(emitFormats)
+        """regex matching names of formats
+        to actually emit interfaces for."""
+
         self.reparentEnums = reparentEnums
         """boolean specifying whether to remove <enum> elements from
         <feature> or <extension> when extending an <enums> type."""
@@ -230,6 +245,10 @@ class GeneratorOptions:
         self.codeGenerator = False
         """True if this generator makes compilable code"""
 
+        self.requireCommandAliases = requireCommandAliases
+        """True if alias= attributes of <command> tags are transitively
+        required."""
+
     def emptyRegex(self, pat):
         """Substitute a regular expression which matches no version
         or extension names for None or the empty string."""
@@ -257,6 +276,17 @@ class OutputGenerator:
         'basetype': 'basetypes',
     }
 
+    def breakName(self, name, msg):
+        """Break into debugger if this is a special name"""
+
+        # List of string names to break on
+        bad = (
+        )
+
+        if name in bad and True:
+            print('breakName {}: {}'.format(name, msg))
+            pdb.set_trace()
+
     def __init__(self, errFile=sys.stderr, warnFile=sys.stderr, diagFile=sys.stdout):
         """Constructor
 
@@ -337,7 +367,7 @@ class OutputGenerator:
             # print('About to translate value =', value, 'type =', type(value))
             if needsNum:
                 numVal = int(value, 0)
-            # If there's a non-integer, numeric 'type' attribute (e.g. 'u' or
+            # If there is a non-integer, numeric 'type' attribute (e.g. 'u' or
             # 'ull'), append it to the string value.
             # t = enuminfo.elem.get('type')
             # if t is not None and t != '' and t != 'i' and t != 's':
@@ -418,7 +448,7 @@ class OutputGenerator:
                                 + ') found with different values:' + strVal
                                 + ' and ' + strVal2)
 
-                # Don't add the duplicate to the returned list
+                # Do not add the duplicate to the returned list
                 continue
             elif numVal in valueMap:
                 # Duplicate value found (such as an alias); report it, but
@@ -519,7 +549,7 @@ class OutputGenerator:
 
         # Accumulate non-numeric enumerant values separately and append
         # them following the numeric values, to allow for aliases.
-        # NOTE: this doesn't do a topological sort yet, so aliases of
+        # NOTE: this does not do a topological sort yet, so aliases of
         # aliases can still get in the wrong order.
         aliasText = ''
 
@@ -536,13 +566,13 @@ class OutputGenerator:
                 self.logMsg('error', 'Allowable range for flag types in C is [', minValidValue, ',', maxValidValue, '], but', name, 'flag has a value outside of this (', strVal, ')\n')
                 exit(1)
 
-            protect = elem.get('protect')
-            if protect is not None:
-                body += '#ifdef {}\n'.format(protect)
-
             decl = self.genRequirements(name, mustBeFound = False)
 
             if self.isEnumRequired(elem):
+                protect = elem.get('protect')
+                if protect is not None:
+                    body += '#ifdef {}\n'.format(protect)
+
                 if usedefine:
                     decl += "#define {} {}\n".format(name, strVal)
                 elif self.misracppstyle():
@@ -553,7 +583,10 @@ class OutputGenerator:
                     # Work around this by chasing the aliases to get the actual value.
                     while numVal is None:
                         alias = self.registry.tree.find("enums/enum[@name='" + strVal + "']")
-                        (numVal, strVal) = self.enumToValue(alias, True)
+                        if alias is not None:
+                            (numVal, strVal) = self.enumToValue(alias, True, bitwidth, True)
+                        else:
+                            self.logMsg('error', 'No such alias {} for enum {}'.format(strVal, name))
                     decl += "static const {} {} = {};\n".format(flagTypeName, name, strVal)
 
                 if numVal is not None:
@@ -561,41 +594,12 @@ class OutputGenerator:
                 else:
                     aliasText += decl
 
-            if protect is not None:
-                body += '#endif\n'
+                if protect is not None:
+                    body += '#endif\n'
 
         # Now append the non-numeric enumerant values
         body += aliasText
 
-        # Generate a range-padding value to ensure the enum is 32 bits, but
-        # only in code generators, so it doesn't appear in documentation.
-        # This isn't needed for bitmasks or defines, but keep them around for
-        # compatibility.
-        if (self.genOpts.codeGenerator or
-            self.conventions.generate_max_enum_in_docs):
-
-            # Break the group name into prefix and suffix portions for range
-            # enum generation
-            expandName = re.sub(r'([0-9a-z_])([A-Z0-9])', r'\1_\2', groupName).upper()
-            expandPrefix = expandName
-            expandSuffix = ''
-            expandSuffixMatch = re.search(r'[A-Z][A-Z]+$', groupName)
-            if expandSuffixMatch:
-                expandSuffix = '_' + expandSuffixMatch.group()
-                # Strip off the suffix from the prefix
-                expandPrefix = expandName.rsplit(expandSuffix, 1)[0]
-
-            maxEnum = '0x7FFFFFFFU'
-            if bitwidth == 64:
-              maxEnum = '0x7FFFFFFFFFFFFFFFULL'
-
-            if usedefine:
-                body += "#define {}_MAX_ENUM{} {}\n".format(expandPrefix, expandSuffix, maxEnum)
-            elif self.misracppstyle():
-                body += "static constexpr {} {}_MAX_ENUM{} {{{}}};\n".format(flagTypeName, expandPrefix, expandSuffix, maxEnum)
-            else:
-                body += "static const {} {}_MAX_ENUM{} = {};\n".format(flagTypeName, expandPrefix, expandSuffix, maxEnum)
-
         # Postfix
 
         return ("bitmask", body)
@@ -641,7 +645,7 @@ class OutputGenerator:
 
         # Accumulate non-numeric enumerant values separately and append
         # them following the numeric values, to allow for aliases.
-        # NOTE: this doesn't do a topological sort yet, so aliases of
+        # NOTE: this does not do a topological sort yet, so aliases of
         # aliases can still get in the wrong order.
         aliasText = []
 
@@ -680,7 +684,7 @@ class OutputGenerator:
                 self.logMsg('error', 'Allowable range for C enum types is [', minValidValue, ',', maxValidValue, '], but', name, 'has a value outside of this (', strVal, ')\n')
                 exit(1)
 
-            # Don't track min/max for non-numbers (numVal is None)
+            # Do not track min/max for non-numbers (numVal is None)
             if isEnum and numVal is not None and elem.get('extends') is None:
                 if minName is None:
                     minName = maxName = name
@@ -703,7 +707,7 @@ class OutputGenerator:
                          "    {}_RANGE_SIZE{} = ({} - {} + 1),".format(expandPrefix, expandSuffix, maxName, minName)))
 
         # Generate a range-padding value to ensure the enum is 32 bits, but
-        # only in code generators, so it doesn't appear in documentation
+        # only in code generators, so it does not appear in documentation
         if (self.genOpts.codeGenerator or
             self.conventions.generate_max_enum_in_docs):
             body.append("    {}_MAX_ENUM{} = 0x7FFFFFFF".format(
@@ -720,6 +724,47 @@ class OutputGenerator:
 
         return (section, '\n'.join(body))
 
+    def buildConstantCDecl(self, enuminfo, name, alias):
+        """Generate the C declaration for a constant (a single <enum>
+        value).
+
+        <enum> tags may specify their values in several ways, but are
+        usually just integers or floating-point numbers."""
+
+        (_, strVal) = self.enumToValue(enuminfo.elem, False)
+
+        if self.misracppstyle() and enuminfo.elem.get('type') and not alias:
+            # Generate e.g.: static constexpr uint32_t x = ~static_cast<uint32_t>(1U);
+            # This appeases MISRA "underlying type" rules.
+            typeStr = enuminfo.elem.get('type');
+            invert = '~' in strVal
+            number = strVal.strip("()~UL")
+            if typeStr != "float":
+                number += 'U'
+            strVal = "~" if invert else ""
+            strVal += "static_cast<" + typeStr + ">(" + number + ")"
+            body = 'static constexpr ' + typeStr.ljust(9) + name.ljust(33) + ' {' + strVal + '};'
+        elif enuminfo.elem.get('type') and not alias:
+            # Generate e.g.: #define x (~0ULL)
+            typeStr = enuminfo.elem.get('type');
+            invert = '~' in strVal
+            paren = '(' in strVal
+            number = strVal.strip("()~UL")
+            if typeStr != "float":
+                if typeStr == "uint64_t":
+                    number += 'ULL'
+                else:
+                    number += 'U'
+            strVal = "~" if invert else ""
+            strVal += number
+            if paren:
+                strVal = "(" + strVal + ")";
+            body = '#define ' + name.ljust(33) + ' ' + strVal;
+        else:
+            body = '#define ' + name.ljust(33) + ' ' + strVal
+
+        return body
+
     def makeDir(self, path):
         """Create a directory, if not already done.
 
@@ -735,7 +780,7 @@ class OutputGenerator:
     def beginFile(self, genOpts):
         """Start a new interface file
 
-        - genOpts - GeneratorOptions controlling what's generated and how"""
+        - genOpts - GeneratorOptions controlling what is generated and how"""
         self.genOpts = genOpts
         self.should_insert_may_alias_macro = \
             self.genOpts.conventions.should_insert_may_alias_macro(self.genOpts)
@@ -766,7 +811,6 @@ class OutputGenerator:
             self.warnFile.flush()
         if self.diagFile:
             self.diagFile.flush()
-        self.outFile.flush()
         if self.outFile != sys.stdout and self.outFile != sys.stderr:
             self.outFile.close()
 
@@ -788,7 +832,7 @@ class OutputGenerator:
         - emit - actually write to the header only when True"""
         self.emit = emit
         self.featureName = interface.get('name')
-        # If there's an additional 'protect' attribute in the feature, save it
+        # If there is an additional 'protect' attribute in the feature, save it
         self.featureExtraProtect = interface.get('protect')
 
     def endFeature(self):
@@ -800,7 +844,7 @@ class OutputGenerator:
 
     def genRequirements(self, name, mustBeFound = True):
         """Generate text showing what core versions and extensions introduce
-        an API. This exists in the base Generator class because it's used by
+        an API. This exists in the base Generator class because it is used by
         the shared enumerant-generating interfaces (buildEnumCDecl, etc.).
         Here it returns an empty string for most generators, but can be
         overridden by e.g. DocGenerator.
@@ -813,7 +857,7 @@ class OutputGenerator:
         return ''
 
     def validateFeature(self, featureType, featureName):
-        """Validate we're generating something only inside a `<feature>` tag"""
+        """Validate we are generating something only inside a `<feature>` tag"""
         if self.featureName is None:
             raise UserWarning('Attempt to generate', featureType,
                               featureName, 'when not in feature')
@@ -875,6 +919,14 @@ class OutputGenerator:
         Extend to generate as desired in your derived class."""
         return
 
+    def genFormat(self, format, formatinfo, alias):
+        """Generate interface for a format element.
+
+        - formatinfo - FormatInfo
+
+        Extend to generate as desired in your derived class."""
+        return
+
     def makeProtoName(self, name, tail):
         """Turn a `<proto>` `<name>` into C-language prototype
         and typedef declarations for that name.
@@ -927,6 +979,9 @@ class OutputGenerator:
 
             # Clear prefix for subsequent iterations
             prefix = ''
+
+        paramdecl = paramdecl + prefix
+
         if aligncol == 0:
             # Squeeze out multiple spaces other than the indentation
             paramdecl = indent + ' '.join(paramdecl.split())
@@ -1001,7 +1056,7 @@ class OutputGenerator:
         return None
 
     def isStructAlwaysValid(self, structname):
-        """Try to do check if a structure is always considered valid (i.e. there's no rules to its acceptance)."""
+        """Try to do check if a structure is always considered valid (i.e. there is no rules to its acceptance)."""
         # A conventions object is required for this call.
         if not self.conventions:
             raise RuntimeError("To use isStructAlwaysValid, be sure your options include a Conventions object.")
@@ -1014,7 +1069,8 @@ class OutputGenerator:
             return False
 
         info = self.registry.typedict.get(structname)
-        assert(info is not None)
+        if info is None:
+            self.logMsg('error', f'isStructAlwaysValid({structname}) - structure not found in typedict')
 
         members = info.getMembers()
 
@@ -1097,7 +1153,7 @@ class OutputGenerator:
         # Leading text
         pdecl += noneStr(proto.text)
         tdecl += noneStr(proto.text)
-        # For each child element, if it's a <name> wrap in appropriate
+        # For each child element, if it is a <name> wrap in appropriate
         # declaration. Otherwise append its contents and tail contents.
         for elem in proto:
             text = noneStr(elem.text)
